@@ -20,8 +20,7 @@ def erc20(get_contract):
     with open("examples/tokens/ERC20.vy") as f:
         contract_code = f.read()
     return get_contract(
-        contract_code,
-        *[TOKEN_NAME, TOKEN_SYMBOL, TOKEN_DECIMALS, TOKEN_INITIAL_SUPPLY],
+        contract_code, *[TOKEN_NAME, TOKEN_SYMBOL, TOKEN_DECIMALS, TOKEN_INITIAL_SUPPLY]
     )
 
 
@@ -36,9 +35,7 @@ def test_initiate(w3, market_maker, erc20, assert_tx_failed):
     a0 = w3.eth.accounts[0]
     erc20.approve(market_maker.address, w3.toWei(2, "ether"), transact={})
     market_maker.initiate(
-        erc20.address,
-        w3.toWei(1, "ether"),
-        transact={"value": w3.toWei(2, "ether")},
+        erc20.address, w3.toWei(1, "ether"), transact={"value": w3.toWei(2, "ether")}
     )
     assert market_maker.totalEthQty() == w3.toWei(2, "ether")
     assert market_maker.totalTokenQty() == w3.toWei(1, "ether")
@@ -59,9 +56,7 @@ def test_eth_to_tokens(w3, market_maker, erc20):
     a1 = w3.eth.accounts[1]
     erc20.approve(market_maker.address, w3.toWei(2, "ether"), transact={})
     market_maker.initiate(
-        erc20.address,
-        w3.toWei(1, "ether"),
-        transact={"value": w3.toWei(2, "ether")},
+        erc20.address, w3.toWei(1, "ether"), transact={"value": w3.toWei(2, "ether")}
     )
     assert erc20.balanceOf(market_maker.address) == w3.toWei(1, "ether")
     assert erc20.balanceOf(a1) == 0
@@ -82,9 +77,7 @@ def test_tokens_to_eth(w3, market_maker, erc20):
     erc20.transfer(a1, w3.toWei(2, "ether"), transact={})
     erc20.approve(market_maker.address, w3.toWei(2, "ether"), transact={"from": a1})
     market_maker.initiate(
-        erc20.address,
-        w3.toWei(1, "ether"),
-        transact={"value": w3.toWei(2, "ether"), "from": a1},
+        erc20.address, w3.toWei(1, "ether"), transact={"value": w3.toWei(2, "ether"), "from": a1}
     )
     assert w3.eth.get_balance(market_maker.address) == w3.toWei(2, "ether")
     # sent 2 eth, with initiate.
@@ -109,9 +102,7 @@ def test_owner_withdraw(w3, market_maker, erc20, assert_tx_failed):
     erc20.approve(market_maker.address, w3.toWei(2, "ether"), transact={})
     # Initiate market with 2 eth value.
     market_maker.initiate(
-        erc20.address,
-        w3.toWei(1, "ether"),
-        transact={"value": w3.toWei(2, "ether")},
+        erc20.address, w3.toWei(1, "ether"), transact={"value": w3.toWei(2, "ether")}
     )
     # 2 eth was sent to market_maker contract.
     assert w3.eth.get_balance(a0) == a0_balance_before - w3.toWei(2, "ether")
