@@ -1,15 +1,3 @@
-def test_block_number(get_contract_with_gas_estimation, w3):
-    w3.testing.mine(1)
-
-    block_number_code = """
-@external
-def block_number() -> uint256:
-    return block.number
-"""
-    c = get_contract_with_gas_estimation(block_number_code)
-    assert c.block_number() == 2
-
-
 def test_blockhash(get_contract_with_gas_estimation, w3):
     w3.testing.mine(1)
 
