@@ -533,6 +533,35 @@ struct X:
 x: X
     """,
     """
+struct X:
+    x: int128
+    y: int128
+struct A:
+    a: X
+    b: uint256
+struct C:
+    c: A
+    d: bool
+@external
+def get_y() -> int128:
+    return C({c: A({a: X({x: 1, y: -1}), b: 777}), d: True}).c.a.y - 10
+    """,
+    """
+struct X:
+    x: int128
+    y: int128
+struct A:
+    a: X
+    b: uint256
+struct C:
+    c: A
+    d: bool
+FOO: constant(C) = C({c: A({a: X({x: 1, y: -1}), b: 777}), d: True})
+@external
+def get_y() -> int128:
+    return FOO.c.a.y - 10
+    """,
+    """
 struct C:
     a: uint256
     b: uint256
