@@ -6,11 +6,11 @@ from vyper.exceptions import StructureException
 FAILING_CONTRACTS = [
     """
 @external
-@view
+@pure
 @nonreentrant('lock')
 def nonreentrant_foo() -> uint256:
     return 1
-    """,
+    """
 ]
 
 
