@@ -30,14 +30,14 @@ def generate_public_variable_getters(vyper_module: vy_ast.Module) -> None:
         Top-level Vyper AST node.
     """
 
-    for node in vyper_module.get_children(vy_ast.AnnAssign, {"annotation.func.id": "public"}):
+    for node in vyper_module.get_children(vy_ast.VariableDecl, {"annotation.func.id": "public"}):
         func_type = node._metadata["func_type"]
         input_types, return_type = func_type.get_signature()
         input_nodes = []
 
         # use the annotation node as a base to build the input args and return type
         # starting with `args[0]` to remove the surrounding `public()` call`
-        annotation = copy.deepcopy(node.annotation.args[0])
+        annotation = copy.copy(node.annotation.args[0])
 
         # the base return statement is an `Attribute` node, e.g. `self.<var_name>`
         # for each input type we wrap it in a `Subscript` to access a specific member
@@ -53,6 +53,9 @@ def generate_public_variable_getters(vyper_module: vy_ast.Module) -> None:
             if annotation.value.get("id") == "HashMap":  # type: ignore
                 # for a HashMap, split the key/value types and use the key type as the next arg
                 arg, annotation = annotation.slice.value.elements  # type: ignore
+            elif annotation.value.get("id") == "DynArray":
+                arg = vy_ast.Name(id=type_._id)
+                annotation = annotation.slice.value.elements[0]  # type: ignore
             else:
                 # for other types, build an input arg node from the expected type
                 # and remove the outer `Subscript` from the annotation
@@ -66,26 +69,20 @@ def generate_public_variable_getters(vyper_module: vy_ast.Module) -> None:
             )
 
         # after iterating the input types, the remaining annotation node is our return type
-        return_node = annotation
-        if isinstance(return_node, vy_ast.Name) and return_node.id != return_type._id:
-            # special case when the return type is an interface
-            # TODO allow interfaces as return types and remove this
-            return_node.id = return_type._id
+        return_node = copy.copy(annotation)
 
         # join everything together as a new `FunctionDef` node, annotate it
         # with the type, and append it to the existing `Module` node
         expanded = vy_ast.FunctionDef.from_node(
             node.annotation,
             name=func_type.name,
-            args=vy_ast.arguments(
-                args=input_nodes,
-                defaults=[],
-            ),
+            args=vy_ast.arguments(args=input_nodes, defaults=[]),
             body=[vy_ast.Return(value=return_stmt)],
             decorator_list=[vy_ast.Name(id="external"), vy_ast.Name(id="view")],
             returns=return_node,
         )
         expanded._metadata["type"] = func_type
+        return_node.set_parent(expanded)
         vyper_module.add_to_body(expanded)
 
 
@@ -103,7 +100,7 @@ def remove_unused_statements(vyper_module: vy_ast.Module) -> None:
     """
 
     # constant declarations - values were substituted within the AST during folding
-    for node in vyper_module.get_children(vy_ast.AnnAssign, {"annotation.func.id": "constant"}):
+    for node in vyper_module.get_children(vy_ast.VariableDecl, {"annotation.func.id": "constant"}):
         vyper_module.remove_from_body(node)
 
     # `implements: interface` statements - validated during type checking
