@@ -1,4 +1,4 @@
-import copy
+import warnings
 from typing import Union
 
 from vyper.ast import nodes as vy_ast
@@ -12,13 +12,14 @@ BUILTIN_CONSTANTS = {
     "EMPTY_BYTES32": (
         vy_ast.Hex,
         "0x0000000000000000000000000000000000000000000000000000000000000000",
+        "empty(bytes32)",
     ),  # NOQA: E501
-    "ZERO_ADDRESS": (vy_ast.Hex, "0x0000000000000000000000000000000000000000"),
-    "MAX_INT128": (vy_ast.Int, 2 ** 127 - 1),
-    "MIN_INT128": (vy_ast.Int, -(2 ** 127)),
-    "MAX_DECIMAL": (vy_ast.Decimal, SizeLimits.MAX_AST_DECIMAL),
-    "MIN_DECIMAL": (vy_ast.Decimal, SizeLimits.MIN_AST_DECIMAL),
-    "MAX_UINT256": (vy_ast.Int, 2 ** 256 - 1),
+    "ZERO_ADDRESS": (vy_ast.Hex, "0x0000000000000000000000000000000000000000", "empty(address)"),
+    "MAX_INT128": (vy_ast.Int, 2 ** 127 - 1, "max_value(int128)"),
+    "MIN_INT128": (vy_ast.Int, -(2 ** 127), "min_value(int128)"),
+    "MAX_DECIMAL": (vy_ast.Decimal, SizeLimits.MAX_AST_DECIMAL, "max_value(decimal)"),
+    "MIN_DECIMAL": (vy_ast.Decimal, SizeLimits.MIN_AST_DECIMAL, "min_value(decimal)"),
+    "MAX_UINT256": (vy_ast.Int, 2 ** 256 - 1, "max_value(uint256)"),
 }
 
 
@@ -146,8 +147,10 @@ def replace_builtin_constants(vyper_module: vy_ast.Module) -> None:
     vyper_module : Module
         Top-level Vyper AST node.
     """
-    for name, (node, value) in BUILTIN_CONSTANTS.items():
-        replace_constant(vyper_module, name, node(value=value), True)  # type: ignore
+    for name, (node, value, replacement) in BUILTIN_CONSTANTS.items():
+        found = replace_constant(vyper_module, name, node(value=value), True)
+        if found > 0:
+            warnings.warn(f"{name} is deprecated. Please use `{replacement}` instead.")
 
 
 def replace_user_defined_constants(vyper_module: vy_ast.Module) -> int:
@@ -167,7 +170,7 @@ def replace_user_defined_constants(vyper_module: vy_ast.Module) -> int:
     """
     changed_nodes = 0
 
-    for node in vyper_module.get_children(vy_ast.AnnAssign):
+    for node in vyper_module.get_children(vy_ast.VariableDecl):
         if not isinstance(node.target, vy_ast.Name):
             # left-hand-side of assignment is not a variable
             continue
@@ -278,14 +281,15 @@ def replace_constant(
 
         if not node.get_ancestor(vy_ast.Index):
             # do not replace left-hand side of assignments
-            assign = node.get_ancestor((vy_ast.Assign, vy_ast.AnnAssign, vy_ast.AugAssign))
+            assign = node.get_ancestor(
+                (vy_ast.Assign, vy_ast.AnnAssign, vy_ast.AugAssign, vy_ast.VariableDecl)
+            )
 
             if assign and node in assign.target.get_descendants(include_self=True):
                 continue
 
         try:
-            # Codegen may mutate AST (particularly structs).
-            replacement_node = copy.deepcopy(replacement_node)
+            # note: _replace creates a copy of the replacement_node
             new_node = _replace(node, replacement_node, type_=type_)
         except UnfoldableNode:
             if raise_on_error:
