@@ -1,4 +1,5 @@
 import ast as python_ast
+import copy
 import decimal
 import operator
 import sys
@@ -17,14 +18,7 @@ from vyper.exceptions import (
 )
 from vyper.utils import MAX_DECIMAL_PLACES, SizeLimits, annotate_source_code
 
-NODE_BASE_ATTRIBUTES = (
-    "_children",
-    "_depth",
-    "_parent",
-    "ast_type",
-    "node_id",
-    "_metadata",
-)
+NODE_BASE_ATTRIBUTES = ("_children", "_depth", "_parent", "ast_type", "node_id", "_metadata")
 NODE_SRC_ATTRIBUTES = (
     "col_offset",
     "end_col_offset",
@@ -62,15 +56,30 @@ def get_node(
     if not isinstance(ast_struct, dict):
         ast_struct = ast_struct.__dict__
 
+        # workaround: some third party module (ex. ipython) might insert
+        # a "parent" member into the node, creating a duplicate kwarg
+        # error below when calling vy_class()
+        if "parent" in ast_struct:
+            ast_struct = copy.copy(ast_struct)
+            del ast_struct["parent"]
+
+    # Replace state and local variable declarations `AnnAssign` with `VariableDecl`
+    # Parent node is required for context to determine whether replacement should happen.
+    if (
+        ast_struct["ast_type"] == "AnnAssign"
+        and isinstance(parent, Module)
+        and not getattr(ast_struct["target"], "id", None) in ("implements",)
+    ):
+        ast_struct["ast_type"] = "VariableDecl"
+
     vy_class = getattr(sys.modules[__name__], ast_struct["ast_type"], None)
     if not vy_class:
         if ast_struct["ast_type"] == "Delete":
             _raise_syntax_exc("Deleting is not supported", ast_struct)
         elif ast_struct["ast_type"] in ("ExtSlice", "Slice"):
             _raise_syntax_exc("Vyper does not support slicing", ast_struct)
-        elif ast_struct["ast_type"] in ("Invert", "UAdd"):
-            op = "+" if ast_struct["ast_type"] == "UAdd" else "~"
-            _raise_syntax_exc(f"Vyper does not support {op} as a unary operator", parent)
+        elif ast_struct["ast_type"] == "UAdd":
+            _raise_syntax_exc("Vyper does not support + as a unary operator", parent)
         else:
             _raise_syntax_exc(
                 f"Invalid syntax (unsupported '{ast_struct['ast_type']}' Python AST node)",
@@ -164,8 +173,7 @@ def _sort_nodes(node_iterable):
         return float("inf") if key is None else key
 
     return sorted(
-        node_iterable,
-        key=lambda k: (sortkey(k.lineno), sortkey(k.col_offset), k.node_id),
+        node_iterable, key=lambda k: (sortkey(k.lineno), sortkey(k.col_offset), k.node_id)
     )
 
 
@@ -240,8 +248,7 @@ class VyperNode:
         **kwargs : dict
             Dictionary of fields to be included within the node.
         """
-        self._parent = parent
-        self._depth = getattr(parent, "_depth", -1) + 1
+        self.set_parent(parent)
         self._children: set = set()
         self._metadata: dict = {}
 
@@ -277,6 +284,11 @@ class VyperNode:
         if parent is not None:
             parent._children.add(self)
 
+    # set parent, can be useful when inserting copied nodes into the AST
+    def set_parent(self, parent: "VyperNode"):
+        self._parent = parent
+        self._depth = getattr(parent, "_depth", -1) + 1
+
     @classmethod
     def from_node(cls, node: "VyperNode", **kwargs) -> "VyperNode":
         """
@@ -389,6 +401,10 @@ class VyperNode:
                 ast_dict[key] = [_to_dict(i) for i in value]
             else:
                 ast_dict[key] = _to_dict(value)
+
+        if "type" in self._metadata:
+            ast_dict["type"] = str(self._metadata["type"])
+
         return ast_dict
 
     def get_ancestor(self, node_type: Union["VyperNode", tuple, None] = None) -> "VyperNode":
@@ -521,6 +537,7 @@ class VyperNode:
 
     def get(self, field_str: str) -> Any:
         """
+
         Recursive getter function for node attributes.
 
         Parameters
@@ -575,8 +592,7 @@ class Module(TopLevel):
         Parameters
         ----------
         old_node : VyperNode
-            Node object to be replaced. If the node does not currently exist
-            within the AST, a `CompilerPanic` is raised.
+            Node object to be replaced.
         new_node : VyperNode
             Node object to replace new_node.
 
@@ -585,8 +601,6 @@ class Module(TopLevel):
         None
         """
         parent = old_node._parent
-        if old_node not in self.get_descendants(type(old_node)):
-            raise CompilerPanic("Node to be replaced does not exist within the tree")
 
         if old_node not in parent._children:
             raise CompilerPanic("Node to be replaced does not exist within parent children")
@@ -682,6 +696,10 @@ class Log(VyperNode):
     __slots__ = ("value",)
 
 
+class EnumDef(VyperNode):
+    __slots__ = ("name", "body")
+
+
 class EventDef(VyperNode):
     __slots__ = ("name", "body")
 
@@ -847,10 +865,7 @@ class Expr(VyperNode):
 
 
 class UnaryOp(VyperNode):
-    __slots__ = (
-        "op",
-        "operand",
-    )
+    __slots__ = ("op", "operand")
 
     def evaluate(self) -> VyperNode:
         """
@@ -865,6 +880,8 @@ class UnaryOp(VyperNode):
             raise UnfoldableNode("Node contains invalid field(s) for evaluation")
         if isinstance(self.op, USub) and not isinstance(self.operand, (Int, Decimal)):
             raise UnfoldableNode("Node contains invalid field(s) for evaluation")
+        if isinstance(self.op, Invert) and not isinstance(self.operand, Int):
+            raise UnfoldableNode("Node contains invalid field(s) for evaluation")
 
         value = self.op._op(self.operand.value)
         _validate_numeric_bounds(self, value)
@@ -882,12 +899,15 @@ class Not(VyperNode):
     _op = operator.not_
 
 
+class Invert(VyperNode):
+    __slots__ = ()
+    _description = "bitwise not"
+    _pretty = "~"
+    _op = operator.inv
+
+
 class BinOp(VyperNode):
-    __slots__ = (
-        "left",
-        "op",
-        "right",
-    )
+    __slots__ = ("left", "op", "right")
 
     def evaluate(self) -> VyperNode:
         """
@@ -995,11 +1015,29 @@ class Pow(VyperNode):
         return int(left ** right)
 
 
+class BitAnd(VyperNode):
+    __slots__ = ()
+    _description = "bitwise and"
+    _pretty = "&"
+    _op = operator.and_
+
+
+class BitOr(VyperNode):
+    __slots__ = ()
+    _description = "bitwise or"
+    _pretty = "|"
+    _op = operator.or_
+
+
+class BitXor(VyperNode):
+    __slots__ = ()
+    _description = "bitwise xor"
+    _pretty = "^"
+    _op = operator.xor
+
+
 class BoolOp(VyperNode):
-    __slots__ = (
-        "op",
-        "values",
-    )
+    __slots__ = ("op", "values")
 
     def evaluate(self) -> VyperNode:
         """
@@ -1023,13 +1061,13 @@ class BoolOp(VyperNode):
 
 class And(VyperNode):
     __slots__ = ()
-    _description = "greater-or-equal"
+    _description = "logical and"
     _op = all
 
 
 class Or(VyperNode):
     __slots__ = ()
-    _description = "less-or-equal"
+    _description = "logical or"
     _op = any
 
 
@@ -1070,6 +1108,8 @@ class Compare(VyperNode):
         if not isinstance(left, Constant):
             raise UnfoldableNode("Node contains invalid field(s) for evaluation")
 
+        # CMC 2022-08-04 we could probably remove these evaluation rules as they
+        # are taken care of in the IR optimizer now.
         if isinstance(self.op, (In, NotIn)):
             if not isinstance(right, List):
                 raise UnfoldableNode("Node contains invalid field(s) for evaluation")
@@ -1151,10 +1191,7 @@ class keyword(VyperNode):
 
 
 class Attribute(VyperNode):
-    __slots__ = (
-        "attr",
-        "value",
-    )
+    __slots__ = ("attr", "value")
 
 
 class Subscript(VyperNode):
@@ -1214,6 +1251,56 @@ class AnnAssign(VyperNode):
     __slots__ = ("target", "annotation", "value", "simple")
 
 
+class VariableDecl(VyperNode):
+    """
+    A contract variable declaration.
+
+    Excludes `simple` attribute from Python `AnnAssign` node.
+
+    Attributes
+    ----------
+    target : VyperNode
+        Left-hand side of the assignment.
+    value : VyperNode
+        Right-hand side of the assignment.
+    annotation : VyperNode
+        Type of variable.
+    is_constant : bool, optional
+        If true, indicates that the variable is a constant variable.
+    is_public : bool, optional
+        If true, indicates that the variable is a public state variable.
+    is_immutable : bool, optional
+        If true, indicates that the variable is an immutable variable.
+    """
+
+    __slots__ = ("target", "annotation", "value", "is_constant", "is_public", "is_immutable")
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        self.is_constant = False
+        self.is_public = False
+        self.is_immutable = False
+
+        if isinstance(self.annotation, Call):
+            # the annotation is a function call, e.g. `foo: constant(uint256)`
+            call_name = self.annotation.get("func.id")
+            if call_name == "constant":
+                # declaring a constant
+                self.is_constant = True
+
+            elif call_name == "public":
+                # declaring a public variable
+                self.is_public = True
+
+            elif call_name == "immutable":
+                # declaring an immutable variable
+                self.is_immutable = True
+
+            else:
+                _raise_syntax_exc("Invalid scope for variable declaration", self.annotation)
+
+
 class AugAssign(VyperNode):
     __slots__ = ("op", "target", "value")
 
