@@ -42,6 +42,7 @@ no-optimize        - Do not optimize (don't use this for production code)
 combined_json_outputs = [
     "bytecode",
     "bytecode_runtime",
+    "blueprint_bytecode",
     "abi",
     "layout",
     "source_map",
@@ -80,27 +81,16 @@ def _parse_args(argv):
         description="Pythonic Smart Contract Language for the EVM",
         formatter_class=argparse.RawTextHelpFormatter,
     )
+    parser.add_argument("input_files", help="Vyper sourcecode to compile", nargs="+")
     parser.add_argument(
-        "input_files",
-        help="Vyper sourcecode to compile",
-        nargs="+",
-    )
-    parser.add_argument(
-        "--version",
-        action="version",
-        version=f"{vyper.__version__}+commit.{vyper.__commit__}",
+        "--version", action="version", version=f"{vyper.__version__}+commit.{vyper.__commit__}"
     )
     parser.add_argument(
         "--show-gas-estimates",
         help="Show gas estimates in abi and ir output mode.",
         action="store_true",
     )
-    parser.add_argument(
-        "-f",
-        help=format_options_help,
-        default="bytecode",
-        dest="format",
-    )
+    parser.add_argument("-f", help=format_options_help, default="bytecode", dest="format")
     parser.add_argument(
         "--storage-layout-file",
         help="Override storage slots provided by compiler",
@@ -114,19 +104,15 @@ def _parse_args(argv):
         default=DEFAULT_EVM_VERSION,
         dest="evm_version",
     )
-    parser.add_argument(
-        "--no-optimize",
-        help="Do not optimize",
-        action="store_true",
-    )
+    parser.add_argument("--no-optimize", help="Do not optimize", action="store_true")
     parser.add_argument(
         "--traceback-limit",
         help="Set the traceback limit for error messages reported by the compiler",
         type=int,
     )
     parser.add_argument(
-        "--debug",
-        help="Turn on compiler debug information. "
+        "--verbose",
+        help="Turn on compiler verbose output. "
         "Currently an alias for --traceback-limit but "
         "may add more information in the future",
         action="store_true",
@@ -136,10 +122,7 @@ def _parse_args(argv):
         help="Switch to standard JSON mode. Use `--standard-json -h` for available options.",
         action="store_true",
     )
-    parser.add_argument(
-        "--hex-ir",
-        action="store_true",
-    )
+    parser.add_argument("--hex-ir", action="store_true")
     parser.add_argument(
         "-p", help="Set the root path for contract imports", default=".", dest="root_folder"
     )
@@ -151,7 +134,7 @@ def _parse_args(argv):
         sys.tracebacklimit = args.traceback_limit
     elif VYPER_TRACEBACK_LIMIT is not None:
         sys.tracebacklimit = VYPER_TRACEBACK_LIMIT
-    elif args.debug:
+    elif args.verbose:
         sys.tracebacklimit = 1000
     else:
         # Python usually defaults sys.tracebacklimit to 1000.  We use a default
@@ -248,10 +231,7 @@ def get_interface_codes(root_path: Path, contract_sources: ContractCodes) -> Dic
                     elif isinstance(contents, list) or (
                         "abi" in contents and isinstance(contents["abi"], list)
                     ):
-                        interfaces[file_path][interface_name] = {
-                            "type": "json",
-                            "code": contents,
-                        }
+                        interfaces[file_path][interface_name] = {"type": "json", "code": contents}
 
                     else:
                         raise ValueError(f"Corrupted file: '{valid_path}'")
