@@ -104,11 +104,21 @@ def build_metadata_output(compiler_data: CompilerData) -> dict:
     warnings.warn("metadata output format is unstable!")
     sigs = compiler_data.function_signatures
 
+    def _var_rec_dict(variable_record):
+        ret = vars(variable_record)
+        ret["typ"] = str(ret["typ"])
+        if ret["data_offset"] is None:
+            del ret["data_offset"]
+        for k in ("blockscopes", "defined_at", "encoding"):
+            del ret[k]
+        ret["location"] = ret["location"].name
+        return ret
+
     def _to_dict(sig):
         ret = vars(sig)
         ret["return_type"] = str(ret["return_type"])
         ret["_ir_identifier"] = sig._ir_identifier
-        for attr in ("gas", "func_ast_code"):
+        for attr in ("gas_estimate", "func_ast_code"):
             del ret[attr]
         for attr in ("args", "base_args", "default_args"):
             if attr in ret:
@@ -116,6 +126,9 @@ def build_metadata_output(compiler_data: CompilerData) -> dict:
         for k in ret["default_values"]:
             # e.g. {"x": vy_ast.Int(..)} -> {"x": 1}
             ret["default_values"][k] = ret["default_values"][k].node_source_code
+        ret["frame_info"] = vars(ret["frame_info"])
+        for k in ret["frame_info"]["frame_vars"].keys():
+            ret["frame_info"]["frame_vars"][k] = _var_rec_dict(ret["frame_info"]["frame_vars"][k])
         return ret
 
     return {"function_info": {name: _to_dict(sig) for (name, sig) in sigs.items()}}
@@ -132,7 +145,7 @@ def build_abi_output(compiler_data: CompilerData) -> list:
     abi = compiler_data.vyper_module_folded._metadata["type"].to_abi_dict()
     if compiler_data.show_gas_estimates:
         # Add gas estimates for each function to ABI
-        gas_estimates = build_gas_estimates(compiler_data.ir_runtime)
+        gas_estimates = build_gas_estimates(compiler_data.function_signatures)
         for func in abi:
             try:
                 func_signature = func["name"]
@@ -143,7 +156,6 @@ def build_abi_output(compiler_data: CompilerData) -> list:
             func_name, _, _ = func_signature.partition("(")
             # This check ensures we skip __init__ since it has no estimate
             if func_name in gas_estimates:
-                # TODO: mutation
                 func["gas"] = gas_estimates[func_name]
     return abi
 
@@ -164,7 +176,7 @@ def _build_asm(asm_list):
     for node in asm_list:
 
         if isinstance(node, list):
-            output_string += "[ " + _build_asm(node) + "] "
+            output_string += "{ " + _build_asm(node) + "} "
             continue
 
         if in_push > 0:
@@ -185,17 +197,16 @@ def _build_asm(asm_list):
 
 
 def build_source_map_output(compiler_data: CompilerData) -> OrderedDict:
-    _, line_number_map = compile_ir.assembly_to_evm(compiler_data.assembly_runtime)
+    _, line_number_map = compile_ir.assembly_to_evm(
+        compiler_data.assembly_runtime, insert_vyper_signature=True
+    )
     # Sort line_number_map
     out = OrderedDict()
     for k in sorted(line_number_map.keys()):
         out[k] = line_number_map[k]
 
     out["pc_pos_map_compressed"] = _compress_source_map(
-        compiler_data.source_code,
-        out["pc_pos_map"],
-        out["pc_jump_map"],
-        compiler_data.source_id,
+        compiler_data.source_code, out["pc_pos_map"], out["pc_jump_map"], compiler_data.source_id
     )
     out["pc_pos_map"] = dict((k, v) for k, v in out["pc_pos_map"].items() if v)
     return out
@@ -203,7 +214,7 @@ def build_source_map_output(compiler_data: CompilerData) -> OrderedDict:
 
 def _compress_source_map(code, pos_map, jump_map, source_id):
     linenos = asttokens.LineNumbers(code)
-    compressed_map = f"-1:-1:{source_id}:-;"
+    ret = [f"-1:-1:{source_id}:-"]
     last_pos = [-1, -1, source_id]
 
     for pc in sorted(pos_map)[1:]:
@@ -223,15 +234,19 @@ def _compress_source_map(code, pos_map, jump_map, source_id):
             else:
                 current_pos[i] = ""
 
-        compressed_map += ":".join(str(i) for i in current_pos) + ";"
+        ret.append(":".join(str(i) for i in current_pos))
 
-    return compressed_map
+    return ";".join(ret)
 
 
 def build_bytecode_output(compiler_data: CompilerData) -> str:
     return f"0x{compiler_data.bytecode.hex()}"
 
 
+def build_blueprint_bytecode_output(compiler_data: CompilerData) -> str:
+    return f"0x{compiler_data.blueprint_bytecode.hex()}"
+
+
 # EIP-170. Ref: https://eips.ethereum.org/EIPS/eip-170
 EIP170_CONTRACT_SIZE_LIMIT: int = 2 ** 14 + 2 ** 13
 
