@@ -1,28 +1,18 @@
 import copy
 import functools
+import math
 
 from vyper.codegen.ir_node import IRnode
 from vyper.evm.opcodes import get_opcodes
 from vyper.exceptions import CodegenPanic, CompilerPanic
 from vyper.utils import MemoryPositions
+from vyper.version import version_tuple
 
 PUSH_OFFSET = 0x5F
 DUP_OFFSET = 0x7F
 SWAP_OFFSET = 0x8F
 
 
-CLAMP_OP_NAMES = {
-    "uclamplt",
-    "uclample",
-    "clamplt",
-    "clample",
-    "uclampgt",
-    "uclampge",
-    "clampgt",
-    "clampge",
-}
-
-
 def num_to_bytearray(x):
     o = []
     while x > 0:
@@ -101,6 +91,13 @@ def _runtime_code_offsets(ctor_mem_size, runtime_codelen):
     return runtime_code_start, runtime_code_end
 
 
+# Calculate the size of PUSH instruction we need to handle all
+# mem offsets in the code. For instance, if we only see mem symbols
+# up to size 256, we can use PUSH1.
+def calc_mem_ofst_size(ctor_mem_size):
+    return math.ceil(math.log(ctor_mem_size + 1, 256))
+
+
 # temporary optimization to handle stack items for return sequences
 # like `return return_ofst return_len`. this is kind of brittle because
 # it assumes the arguments are already on the stack, to be replaced
@@ -140,20 +137,23 @@ def _rewrite_return_sequences(ir_node, label_params=None):
 
 
 def _assert_false():
+    global _revert_label
     # use a shared failure block for common case of assert(x).
     # in the future we might want to change the code
     # at _sym_revert0 to: INVALID
-    return ["_sym_revert0", "JUMPI"]
+    return [_revert_label, "JUMPI"]
 
 
 def _add_postambles(asm_ops):
     to_append = []
 
-    _revert0_string = ["_sym_revert0", "JUMPDEST", "PUSH1", 0, "DUP1", "REVERT"]
+    global _revert_label
 
-    if "_sym_revert0" in asm_ops:
+    _revert_string = [_revert_label, "JUMPDEST", "PUSH1", 0, "DUP1", "REVERT"]
+
+    if _revert_label in asm_ops:
         # shared failure block
-        to_append.extend(_revert0_string)
+        to_append.extend(_revert_string)
 
     if len(to_append) > 0:
         # for some reason there might not be a STOP at the end of asm_ops.
@@ -172,8 +172,10 @@ class Instruction(str):
     def __new__(cls, sstr, *args, **kwargs):
         return super().__new__(cls, sstr)
 
-    def __init__(self, sstr, source_pos=None):
+    def __init__(self, sstr, source_pos=None, error_msg=None):
+        self.error_msg = error_msg
         self.pc_debugger = False
+
         if source_pos is not None:
             self.lineno, self.col_offset, self.end_lineno, self.end_col_offset = source_pos
         else:
@@ -185,8 +187,9 @@ def apply_line_numbers(func):
     def apply_line_no_wrapper(*args, **kwargs):
         code = args[0]
         ret = func(*args, **kwargs)
+
         new_ret = [
-            Instruction(i, code.source_pos)
+            Instruction(i, code.source_pos, code.error_msg)
             if isinstance(i, str) and not isinstance(i, Instruction)
             else i
             for i in ret
@@ -198,6 +201,9 @@ def apply_line_numbers(func):
 
 @apply_line_numbers
 def compile_to_assembly(code, no_optimize=False):
+    global _revert_label
+    _revert_label = mksymbol("revert")
+
     # don't overwrite ir since the original might need to be output, e.g. `-f ir,asm`
     code = copy.deepcopy(code)
     _rewrite_return_sequences(code)
@@ -238,7 +244,7 @@ def _compile_to_assembly(code, withargs=None, existing_labels=None, break_dest=N
     if existing_labels is None:
         existing_labels = set()
     if not isinstance(existing_labels, set):
-        raise CompilerPanic(f"Incorrect type for existing_labels: {type(existing_labels)}")
+        raise CompilerPanic(f"must be set(), but got {type(existing_labels)}")
 
     # Opcodes
     if isinstance(code.value, str) and code.value.upper() in get_opcodes():
@@ -376,15 +382,7 @@ def _compile_to_assembly(code, withargs=None, existing_labels=None, break_dest=N
         )
 
         # stack: []
-        o.extend(
-            _compile_to_assembly(
-                start,
-                withargs,
-                existing_labels,
-                break_dest,
-                height,
-            )
-        )
+        o.extend(_compile_to_assembly(start, withargs, existing_labels, break_dest, height))
 
         o.extend(_compile_to_assembly(rounds, withargs, existing_labels, break_dest, height + 1))
 
@@ -403,7 +401,7 @@ def _compile_to_assembly(code, withargs=None, existing_labels=None, break_dest=N
             # TODO this runtime assertion should never fail for
             # internally generated repeats.
             # maybe drop it or jump to 0xFE
-            o.extend(["DUP2", "GT", "_sym_revert0", "JUMPI"])
+            o.extend(["DUP2", "GT"] + _assert_false())
 
             # stack: i, rounds
             # if (0 == rounds) { goto end_dest; }
@@ -424,11 +422,7 @@ def _compile_to_assembly(code, withargs=None, existing_labels=None, break_dest=N
         o.extend([entry_dest, "JUMPDEST"])
         o.extend(
             _compile_to_assembly(
-                body,
-                withargs,
-                existing_labels,
-                (exit_dest, continue_dest, height + 2),
-                height + 2,
+                body, withargs, existing_labels, (exit_dest, continue_dest, height + 2), height + 2
             )
         )
 
@@ -483,13 +477,7 @@ def _compile_to_assembly(code, withargs=None, existing_labels=None, break_dest=N
         old = withargs.get(code.args[0].value, None)
         withargs[code.args[0].value] = height
         o.extend(
-            _compile_to_assembly(
-                code.args[2],
-                withargs,
-                existing_labels,
-                break_dest,
-                height + 1,
-            )
+            _compile_to_assembly(code.args[2], withargs, existing_labels, break_dest, height + 1)
         )
         if code.args[2].valency:
             o.extend(["SWAP1", "POP"])
@@ -510,7 +498,8 @@ def _compile_to_assembly(code, withargs=None, existing_labels=None, break_dest=N
         assert isinstance(padding, int), "non-int padding"
 
         begincode = mksymbol("runtime_begin")
-        subcode = _compile_to_assembly(ir, {}, existing_labels, None, 0)
+
+        subcode = _compile_to_assembly(ir)
 
         o = []
 
@@ -518,7 +507,7 @@ def _compile_to_assembly(code, withargs=None, existing_labels=None, break_dest=N
         o.extend(["_sym_subcode_size", begincode, "_mem_deploy_start", "CODECOPY"])
 
         # calculate the len of runtime code
-        o.extend(["_sym_subcode_size"] + PUSH(padding) + ["ADD"])  # stack: len
+        o.extend(["_OFST", "_sym_subcode_size", padding])  # stack: len
         o.extend(["_mem_deploy_start"])  # stack: len mem_ofst
         o.extend(["RETURN"])
 
@@ -560,98 +549,7 @@ def _compile_to_assembly(code, withargs=None, existing_labels=None, break_dest=N
         o.extend(["ISZERO"])
         o.extend(_assert_false())
         return o
-    # Unsigned/signed clamp, check less-than
-    elif code.value in CLAMP_OP_NAMES:
-        if isinstance(code.args[0].value, int) and isinstance(code.args[1].value, int):
-            # Checks for clamp errors at compile time as opposed to run time
-            # TODO move these to optimizer.py
-            args_0_val = code.args[0].value
-            args_1_val = code.args[1].value
-            is_free_of_clamp_errors = any(
-                (
-                    code.value in ("uclamplt", "clamplt") and 0 <= args_0_val < args_1_val,
-                    code.value in ("uclample", "clample") and 0 <= args_0_val <= args_1_val,
-                    code.value in ("uclampgt", "clampgt") and 0 <= args_0_val > args_1_val,
-                    code.value in ("uclampge", "clampge") and 0 <= args_0_val >= args_1_val,
-                )
-            )
-            if is_free_of_clamp_errors:
-                return _compile_to_assembly(
-                    code.args[0],
-                    withargs,
-                    existing_labels,
-                    break_dest,
-                    height,
-                )
-            else:
-                raise Exception(
-                    f"Invalid {code.value} with values {code.args[0]} and {code.args[1]}"
-                )
-        o = _compile_to_assembly(code.args[0], withargs, existing_labels, break_dest, height)
-        o.extend(
-            _compile_to_assembly(
-                code.args[1],
-                withargs,
-                existing_labels,
-                break_dest,
-                height + 1,
-            )
-        )
-        o.extend(["DUP2"])
-        # Stack: num num bound
-        if code.value == "uclamplt":
-            o.extend(["LT", "ISZERO"])
-        elif code.value == "clamplt":
-            o.extend(["SLT", "ISZERO"])
-        elif code.value == "uclample":
-            o.extend(["GT"])
-        elif code.value == "clample":
-            o.extend(["SGT"])
-        elif code.value == "uclampgt":
-            o.extend(["GT", "ISZERO"])
-        elif code.value == "clampgt":
-            o.extend(["SGT", "ISZERO"])
-        elif code.value == "uclampge":
-            o.extend(["LT"])
-        elif code.value == "clampge":
-            o.extend(["SLT"])
-        o.extend(_assert_false())
-        return o
-    # Signed clamp, check against upper and lower bounds
-    elif code.value in ("clamp", "uclamp"):
-        comp1 = "SGT" if code.value == "clamp" else "GT"
-        comp2 = "SLT" if code.value == "clamp" else "LT"
-        o = _compile_to_assembly(code.args[0], withargs, existing_labels, break_dest, height)
-        o.extend(
-            _compile_to_assembly(
-                code.args[1],
-                withargs,
-                existing_labels,
-                break_dest,
-                height + 1,
-            )
-        )
-        o.extend(["DUP1"])
-        o.extend(
-            _compile_to_assembly(
-                code.args[2],
-                withargs,
-                existing_labels,
-                break_dest,
-                height + 3,
-            )
-        )
-        o.extend(["SWAP1", comp1])
-        o.extend(_assert_false())
-        o.extend(["DUP1", "SWAP2", "SWAP1", comp2])
-        o.extend(_assert_false())
-        return o
-    # Checks that a value is nonzero
-    elif code.value == "clamp_nonzero":
-        o = _compile_to_assembly(code.args[0], withargs, existing_labels, break_dest, height)
-        o.extend(["DUP1", "ISZERO"])
-        o.extend(_assert_false())
-        return o
+
     # SHA3 a single value
     elif code.value == "sha3_32":
         o = _compile_to_assembly(code.args[0], withargs, existing_labels, break_dest, height)
@@ -814,6 +712,17 @@ def _compile_to_assembly(code, withargs=None, existing_labels=None, break_dest=N
 
         return ["_sym_" + label_name, "JUMPDEST"] + body_asm + pop_scoped_vars
 
+    elif code.value == "unique_symbol":
+        symbol = code.args[0].value
+        assert isinstance(symbol, str)
+
+        if symbol in existing_labels:
+            raise Exception(f"symbol {symbol} already exists!")
+        else:
+            existing_labels.add(symbol)
+
+        return []
+
     elif code.value == "exit_to":
         raise CodegenPanic("exit_to not implemented yet!")
 
@@ -834,7 +743,12 @@ def note_line_num(line_number_map, item, pos):
             offsets = (item.lineno, item.col_offset, item.end_lineno, item.end_col_offset)
         else:
             offsets = None
+
         line_number_map["pc_pos_map"][pos] = offsets
+
+        if item.error_msg is not None:
+            line_number_map["error_map"][pos] = item.error_msg
+
     added_line_breakpoint = note_breakpoint(line_number_map, item, pos)
     return added_line_breakpoint
 
@@ -856,18 +770,23 @@ def _prune_unreachable_code(assembly):
     # end of a function that has already returned or reverted. This should
     # be addressed in the IR, but for now we do a final sanity check here
     # to avoid unnecessary bytecode bloat.
+    changed = False
     i = 0
     while i < len(assembly) - 1:
         if assembly[i] in ("JUMP", "RETURN", "REVERT", "STOP") and not (
             is_symbol(assembly[i + 1]) or assembly[i + 1] == "JUMPDEST"
         ):
+            changed = True
             del assembly[i + 1]
         else:
             i += 1
 
+    return changed
+
 
 def _prune_inefficient_jumps(assembly):
     # prune sequences `_sym_x JUMP _sym_x JUMPDEST` to `_sym_x JUMPDEST`
+    changed = False
     i = 0
     while i < len(assembly) - 4:
         if (
@@ -877,10 +796,13 @@ def _prune_inefficient_jumps(assembly):
             and assembly[i + 3] == "JUMPDEST"
         ):
             # delete _sym_x JUMP
+            changed = True
             del assembly[i : i + 2]
         else:
             i += 1
 
+    return changed
+
 
 def _merge_jumpdests(assembly):
     # When we have multiple JUMPDESTs in a row, or when a JUMPDEST
@@ -888,6 +810,7 @@ def _merge_jumpdests(assembly):
     # intermediate jumps.
     # (Usually a chain of JUMPs is created by a nested block,
     # or some nested if statements.)
+    changed = False
     i = 0
     while i < len(assembly) - 3:
         if is_symbol(assembly[i]) and assembly[i + 1] == "JUMPDEST":
@@ -900,6 +823,7 @@ def _merge_jumpdests(assembly):
                 for j in range(len(assembly)):
                     if assembly[j] == current_symbol and i != j:
                         assembly[j] = new_symbol
+                        changed = True
             elif is_symbol(assembly[i + 2]) and assembly[i + 3] == "JUMP":
                 # _sym_x JUMPDEST _sym_y JUMP
                 # replace all instances of _sym_x with _sym_y
@@ -908,15 +832,41 @@ def _merge_jumpdests(assembly):
                 for j in range(len(assembly)):
                     if assembly[j] == current_symbol and i != j:
                         assembly[j] = new_symbol
+                        changed = True
 
         i += 1
 
+    return changed
+
+
+_RETURNS_ZERO_OR_ONE = {
+    "LT",
+    "GT",
+    "SLT",
+    "SGT",
+    "EQ",
+    "ISZERO",
+    "CALL",
+    "STATICCALL",
+    "CALLCODE",
+    "DELEGATECALL",
+}
+
 
 def _merge_iszero(assembly):
+    changed = False
+
     i = 0
+    # list of opcodes that return 0 or 1
     while i < len(assembly) - 2:
-        if assembly[i : i + 3] == ["ISZERO", "ISZERO", "ISZERO"]:
-            del assembly[i : i + 2]
+        if (
+            isinstance(assembly[i], str)
+            and assembly[i] in _RETURNS_ZERO_OR_ONE
+            and assembly[i + 1 : i + 3] == ["ISZERO", "ISZERO"]
+        ):
+            changed = True
+            # drop the extra iszeros
+            del assembly[i + 1 : i + 3]
         else:
             i += 1
     i = 0
@@ -928,12 +878,17 @@ def _merge_iszero(assembly):
             and is_symbol(assembly[i + 2])
             and assembly[i + 3] == "JUMPI"
         ):
+            changed = True
             del assembly[i : i + 2]
         else:
             i += 1
 
+    return changed
+
 
 def _prune_unused_jumpdests(assembly):
+    changed = False
+
     used_jumpdests = set()
 
     # find all used jumpdests
@@ -945,28 +900,36 @@ def _prune_unused_jumpdests(assembly):
     i = 0
     while i < len(assembly) - 2:
         if is_symbol(assembly[i]) and assembly[i] not in used_jumpdests:
+            changed = True
             del assembly[i : i + 2]
         else:
             i += 1
 
+    return changed
+
 
 def _stack_peephole_opts(assembly):
+    changed = False
     i = 0
     while i < len(assembly) - 2:
         # usually generated by with statements that return their input like
         # (with x (...x))
         if assembly[i : i + 3] == ["DUP1", "SWAP1", "POP"]:
             # DUP1 SWAP1 POP == no-op
+            changed = True
             del assembly[i : i + 3]
             continue
         # usually generated by nested with statements that don't return like
         # (with x (with y ...))
         if assembly[i : i + 3] == ["SWAP1", "POP", "POP"]:
             # SWAP1 POP POP == POP POP
+            changed = True
             del assembly[i]
             continue
         i += 1
 
+    return changed
+
 
 # optimize assembly, in place
 def _optimize_assembly(assembly):
@@ -974,97 +937,170 @@ def _optimize_assembly(assembly):
         if isinstance(x, list):
             _optimize_assembly(x)
 
-    _prune_unreachable_code(assembly)
-    _merge_iszero(assembly)
-    _merge_jumpdests(assembly)
-    _prune_inefficient_jumps(assembly)
-    _prune_unused_jumpdests(assembly)
-    _stack_peephole_opts(assembly)
+    for _ in range(1024):
+        changed = False
+
+        changed |= _prune_unreachable_code(assembly)
+        changed |= _merge_iszero(assembly)
+        changed |= _merge_jumpdests(assembly)
+        changed |= _prune_inefficient_jumps(assembly)
+        changed |= _prune_unused_jumpdests(assembly)
+        changed |= _stack_peephole_opts(assembly)
+
+        if not changed:
+            return
+
+    raise CompilerPanic("infinite loop detected during assembly reduction")  # pragma: notest
 
 
-# Assembles assembly into EVM
-def assembly_to_evm(assembly, start_pos=0):
+def adjust_pc_maps(pc_maps, ofst):
+    assert ofst >= 0
+
+    ret = {}
+    # source breakpoints, don't need to modify
+    ret["breakpoints"] = pc_maps["breakpoints"].copy()
+    ret["pc_breakpoints"] = {pc + ofst for pc in pc_maps["pc_breakpoints"]}
+    ret["pc_jump_map"] = {k + ofst: v for (k, v) in pc_maps["pc_jump_map"].items()}
+    ret["pc_pos_map"] = {k + ofst: v for (k, v) in pc_maps["pc_pos_map"].items()}
+    ret["error_map"] = {k + ofst: v for (k, v) in pc_maps["error_map"].items()}
+
+    return ret
+
+
+def assembly_to_evm(assembly, pc_ofst=0, insert_vyper_signature=False):
+    """
+    Assembles assembly into EVM
+
+    assembly: list of asm instructions
+    pc_ofst: when constructing the source map, the amount to offset all
+             pcs by (no effect until we add deploy code source map)
+    insert_vyper_signature: whether to append vyper metadata to output
+                            (should be true for runtime code)
+    """
     line_number_map = {
         "breakpoints": set(),
         "pc_breakpoints": set(),
         "pc_jump_map": {0: "-"},
         "pc_pos_map": {},
+        "error_map": {},
     }
 
-    posmap = {}
+    pc = 0
+    symbol_map = {}
+
     runtime_code, runtime_code_start, runtime_code_end = None, None, None
-    pos = start_pos
 
-    # go through the code, resolving symbolic locations
-    # (i.e. JUMPDEST locations) to actual code locations
+    bytecode_suffix = b""
+    if insert_vyper_signature:
+        # CBOR encoded: {"vyper": [major,minor,patch]}
+        bytecode_suffix += b"\xa1\x65vyper\x83" + bytes(list(version_tuple))
+        bytecode_suffix += len(bytecode_suffix).to_bytes(2, "big")
+
+    CODE_OFST_SIZE = 2  # size of a PUSH instruction for a code symbol
+
+    # to optimize the size of deploy code - we want to use the smallest
+    # PUSH instruction possible which can support all memory symbols
+    # (and also works with linear pass symbol resolution)
+    # to do this, we first do a single pass to compile any runtime code
+    # and use that to calculate mem_ofst_size.
+    mem_ofst_size, ctor_mem_size = None, None
+    max_mem_ofst = 0
     for i, item in enumerate(assembly):
-        note_line_num(line_number_map, item, pos)
-        if item == "DEBUG":
-            continue  # skip debug
-
-        if item == "JUMP":
-            last = assembly[i - 1]
-            if is_symbol(last) and last.startswith("_sym_internal"):
-                if last.endswith("cleanup"):
-                    # exit an internal function
-                    line_number_map["pc_jump_map"][pos] = "o"
-                else:
-                    # enter an internal function
-                    line_number_map["pc_jump_map"][pos] = "i"
-            else:
-                # everything else
-                line_number_map["pc_jump_map"][pos] = "-"
-        elif item in ("JUMPI", "JUMPDEST"):
-            line_number_map["pc_jump_map"][pos] = "-"
-
-        if is_symbol(item):
-            if assembly[i + 1] == "JUMPDEST" or assembly[i + 1] == "BLANK":
-                # Don't increment position as the symbol itself doesn't go into code
-                if item in posmap:
-                    raise CompilerPanic(f"duplicate jumpdest {item}")
-
-                posmap[item] = pos - start_pos
-            else:
-                pos += 3  # PUSH2 highbits lowbits
-        elif is_mem_sym(item):
-            pos += 5  # PUSH4 item
-        elif is_ofst(item):
-            assert is_symbol(assembly[i + 1]) or is_mem_sym(assembly[i + 1])
-            assert isinstance(assembly[i + 2], int)
-            # [_OFST, _sym_foo, bar] -> PUSH2 (foo+bar)
-            # [_OFST, _mem_foo, bar] -> PUSH4 (foo+bar)
-            pos -= 1
-        elif item == "BLANK":
-            pos += 0
-        elif isinstance(item, str) and item.startswith("_DEPLOY_MEM_OFST_"):
-            # _DEPLOY_MEM_OFST is assembly magic which will
-            # get removed during final assembly-to-bytecode
-            pos += 0
-        elif isinstance(item, list):
+        if isinstance(item, list):
             assert runtime_code is None, "Multiple subcodes"
-            runtime_code, sub_map = assembly_to_evm(item, start_pos=pos)
+            runtime_code, runtime_map = assembly_to_evm(item, insert_vyper_signature=True)
+
             assert item[0].startswith("_DEPLOY_MEM_OFST_")
+            assert ctor_mem_size is None
             ctor_mem_size = int(item[0][len("_DEPLOY_MEM_OFST_") :])
 
             runtime_code_start, runtime_code_end = _runtime_code_offsets(
                 ctor_mem_size, len(runtime_code)
             )
             assert runtime_code_end - runtime_code_start == len(runtime_code)
-            pos += len(runtime_code)
-            for key in line_number_map:
-                line_number_map[key].update(sub_map[key])
-        else:
-            pos += 1
 
-    code_end = pos - start_pos
-    posmap["_sym_code_end"] = code_end
-    posmap["_mem_deploy_start"] = runtime_code_start
-    posmap["_mem_deploy_end"] = runtime_code_end
+        if is_ofst(item) and is_mem_sym(assembly[i + 1]):
+            max_mem_ofst = max(assembly[i + 2], max_mem_ofst)
+
+    if runtime_code_end is not None:
+        mem_ofst_size = calc_mem_ofst_size(runtime_code_end + max_mem_ofst)
+
+    # go through the code, resolving symbolic locations
+    # (i.e. JUMPDEST locations) to actual code locations
+    for i, item in enumerate(assembly):
+        note_line_num(line_number_map, item, pc)
+        if item == "DEBUG":
+            continue  # skip debug
+
+        # update pc_jump_map
+        if item == "JUMP":
+            last = assembly[i - 1]
+            if is_symbol(last) and last.startswith("_sym_internal"):
+                if last.endswith("cleanup"):
+                    # exit an internal function
+                    line_number_map["pc_jump_map"][pc] = "o"
+                else:
+                    # enter an internal function
+                    line_number_map["pc_jump_map"][pc] = "i"
+            else:
+                # everything else
+                line_number_map["pc_jump_map"][pc] = "-"
+        elif item in ("JUMPI", "JUMPDEST"):
+            line_number_map["pc_jump_map"][pc] = "-"
+
+        # update pc
+        if is_symbol(item):
+            if assembly[i + 1] == "JUMPDEST" or assembly[i + 1] == "BLANK":
+                # Don't increment pc as the symbol itself doesn't go into code
+                if item in symbol_map:
+                    raise CompilerPanic(f"duplicate jumpdest {item}")
+
+                symbol_map[item] = pc
+            else:
+                pc += CODE_OFST_SIZE + 1  # PUSH2 highbits lowbits
+        elif is_mem_sym(item):
+            # PUSH<n> item
+            pc += mem_ofst_size + 1
+        elif is_ofst(item):
+            assert is_symbol(assembly[i + 1]) or is_mem_sym(assembly[i + 1])
+            assert isinstance(assembly[i + 2], int)
+            # [_OFST, _sym_foo, bar] -> PUSH2 (foo+bar)
+            # [_OFST, _mem_foo, bar] -> PUSHN (foo+bar)
+            pc -= 1
+        elif item == "BLANK":
+            pc += 0
+        elif isinstance(item, str) and item.startswith("_DEPLOY_MEM_OFST_"):
+            # _DEPLOY_MEM_OFST is assembly magic which will
+            # get removed during final assembly-to-bytecode
+            pc += 0
+        elif isinstance(item, list):
+            # add source map for all items in the runtime map
+            t = adjust_pc_maps(runtime_map, pc)
+            for key in line_number_map:
+                line_number_map[key].update(t[key])
+            pc += len(runtime_code)
+
+        else:
+            pc += 1
+
+    pc += len(bytecode_suffix)
+
+    symbol_map["_sym_code_end"] = pc
+    symbol_map["_mem_deploy_start"] = runtime_code_start
+    symbol_map["_mem_deploy_end"] = runtime_code_end
     if runtime_code is not None:
-        posmap["_sym_subcode_size"] = len(runtime_code)
+        symbol_map["_sym_subcode_size"] = len(runtime_code)
+
+    # (NOTE CMC 2022-06-17 this way of generating bytecode did not
+    # seem to be a perf hotspot. if it is, may want to use bytearray()
+    # instead).
+
+    # TODO refactor into two functions, create posmap and assemble
 
     o = b""
 
+    # now that all symbols have been resolved, generate bytecode
+    # using the symbol map
     to_skip = 0
     for i, item in enumerate(assembly):
         if to_skip > 0:
@@ -1079,17 +1115,17 @@ def assembly_to_evm(assembly, start_pos=0):
 
         elif is_symbol(item):
             if assembly[i + 1] != "JUMPDEST" and assembly[i + 1] != "BLANK":
-                bytecode, _ = assembly_to_evm(PUSH_N(posmap[item], n=2))
+                bytecode, _ = assembly_to_evm(PUSH_N(symbol_map[item], n=CODE_OFST_SIZE))
                 o += bytecode
 
         elif is_mem_sym(item):
-            bytecode, _ = assembly_to_evm(PUSH_N(posmap[item], n=4))
+            bytecode, _ = assembly_to_evm(PUSH_N(symbol_map[item], n=mem_ofst_size))
             o += bytecode
 
         elif is_ofst(item):
             # _OFST _sym_foo 32
-            ofst = posmap[assembly[i + 1]] + assembly[i + 2]
-            n = 4 if is_mem_sym(assembly[i + 1]) else 2
+            ofst = symbol_map[assembly[i + 1]] + assembly[i + 2]
+            n = mem_ofst_size if is_mem_sym(assembly[i + 1]) else CODE_OFST_SIZE
             bytecode, _ = assembly_to_evm(PUSH_N(ofst, n))
             o += bytecode
             to_skip = 2
@@ -1110,7 +1146,8 @@ def assembly_to_evm(assembly, start_pos=0):
             # Should never reach because, assembly is create in _compile_to_assembly.
             raise Exception("Weird symbol in assembly: " + str(item))  # pragma: no cover
 
-    assert len(o) == pos - start_pos, (len(o), pos, start_pos)
+    o += bytecode_suffix
+
     line_number_map["breakpoints"] = list(line_number_map["breakpoints"])
     line_number_map["pc_breakpoints"] = list(line_number_map["pc_breakpoints"])
     return o, line_number_map
