@@ -1,6 +1,7 @@
 from typing import Union
 
 from vyper import ast as vy_ast
+from vyper.abi_types import ABI_Bool, ABIType
 from vyper.exceptions import InvalidLiteral
 
 from ..bases import BasePrimitive, BaseTypeDefinition, ValueTypeDefinition
@@ -19,6 +20,10 @@ class BoolDefinition(ValueTypeDefinition):
             return
         super().validate_numeric_op(node)
 
+    @property
+    def abi_type(self) -> ABIType:
+        return ABI_Bool()
+
 
 class BoolPrimitive(BasePrimitive):
 
