@@ -1,7 +1,11 @@
 import binascii
+import contextlib
 import decimal
+import functools
 import sys
+import time
 import traceback
+import warnings
 from typing import List, Union
 
 from vyper.exceptions import DecimalOverrideException, InvalidLiteral
@@ -10,8 +14,14 @@ from vyper.exceptions import DecimalOverrideException, InvalidLiteral
 class DecimalContextOverride(decimal.Context):
     def __setattr__(self, name, value):
         if name == "prec":
-            # CMC 2022-03-27: should we raise a warning instead of an exception?
-            raise DecimalOverrideException("Overriding decimal precision disabled")
+            if value < 78:
+                # definitely don't want this to happen
+                raise DecimalOverrideException("Overriding decimal precision disabled")
+            elif value > 78:
+                # not sure it's incorrect, might not be end of the world
+                warnings.warn("Changing decimals precision could have unintended side effects!")
+            # else: no-op, is ok
+
         super().__setattr__(name, value)
 
 
@@ -39,30 +49,48 @@ def fourbytes_to_int(inp):
     return (inp[0] << 24) + (inp[1] << 16) + (inp[2] << 8) + inp[3]
 
 
-def signed_to_unsigned(int_, bits):
+def signed_to_unsigned(int_, bits, strict=False):
     """
     Reinterpret a signed integer with n bits as an unsigned integer.
     The implementation is unforgiving in that it assumes the input is in
     bounds for int<bits>, in order to fail more loudly (and not hide
     errors in modular reasoning in consumers of this function).
     """
+    if strict:
+        lo, hi = int_bounds(signed=True, bits=bits)
+        assert lo <= int_ <= hi
     if int_ < 0:
         return int_ + 2 ** bits
     return int_
 
 
-def unsigned_to_signed(int_, bits):
+def unsigned_to_signed(int_, bits, strict=False):
     """
     Reinterpret an unsigned integer with n bits as a signed integer.
     The implementation is unforgiving in that it assumes the input is in
     bounds for uint<bits>, in order to fail more loudly (and not hide
     errors in modular reasoning in consumers of this function).
     """
+    if strict:
+        lo, hi = int_bounds(signed=False, bits=bits)
+        assert lo <= int_ <= hi
     if int_ > (2 ** (bits - 1)) - 1:
         return int_ - (2 ** bits)
     return int_
 
 
+def is_power_of_two(n: int) -> bool:
+    # busted for ints wider than 53 bits:
+    # t = math.log(n, 2)
+    # return math.ceil(t) == math.floor(t)
+    return n != 0 and ((n & (n - 1)) == 0)
+
+
+# https://stackoverflow.com/a/71122440/
+def int_log2(n: int) -> int:
+    return n.bit_length() - 1
+
+
 # utility function for debugging purposes
 def trace(n=5, out=sys.stderr):
     print("BEGIN TRACE", file=out)
@@ -156,6 +184,7 @@ GAS_CALLDATACOPY_WORD = 3
 # A decimal value can store multiples of 1/DECIMAL_DIVISOR
 MAX_DECIMAL_PLACES = 10
 DECIMAL_DIVISOR = 10 ** MAX_DECIMAL_PLACES
+DECIMAL_EPSILON = decimal.Decimal(1) / DECIMAL_DIVISOR
 
 
 def int_bounds(signed, bits):
@@ -169,12 +198,16 @@ def int_bounds(signed, bits):
     return 0, (2 ** bits) - 1
 
 
+# e.g. -1 -> -(2**256 - 1)
+def evm_twos_complement(x: int) -> int:
+    # return ((o + 2 ** 255) % 2 ** 256) - 2 ** 255
+    return ((2 ** 256 - 1) ^ x) + 1
+
+
 # EVM div semantics as a python function
 def evm_div(x, y):
     if y == 0:
         return 0
-    # doesn't actually work:
-    # return int(x / y)
     # NOTE: should be same as: round_towards_zero(Decimal(x)/Decimal(y))
     sign = -1 if (x * y) < 0 else 1
     return sign * (abs(x) // abs(y))  # adapted from py-evm
@@ -185,12 +218,17 @@ def evm_mod(x, y):
     if y == 0:
         return 0
 
-    # this doesn't actually work when num digits exceeds fp precision:
-    # return int(math.fmod(x, y))
     sign = -1 if x < 0 else 1
     return sign * (abs(x) % abs(y))  # adapted from py-evm
 
 
+# EVM pow which wraps instead of hanging on "large" numbers
+# (which can generated, for ex. in the unevaluated branch of the Shift builtin)
+def evm_pow(x, y):
+    assert x >= 0 and y >= 0
+    return pow(x, y, 2 ** 256)
+
+
 # memory used for system purposes, not for variables
 class MemoryPositions:
     FREE_VAR_SPACE = 0
@@ -229,9 +267,7 @@ class SizeLimits:
 
 
 # Otherwise reserved words that are whitelisted for function declarations
-FUNCTION_WHITELIST = {
-    "send",
-}
+FUNCTION_WHITELIST = {"send"}
 
 # List of valid IR macros.
 # TODO move this somewhere else, like ir_node.py
@@ -243,12 +279,6 @@ VALID_IR_MACROS = {
     "dload",
     "dloadbytes",
     "ceil32",
-    "clamp",
-    "clamp_nonzero",
-    "clampge",
-    "clampgt",
-    "clample",
-    "clamplt",
     "continue",
     "debugger",
     "ge",
@@ -265,11 +295,6 @@ VALID_IR_MACROS = {
     "sha3_32",
     "sha3_64",
     "sle",
-    "uclamp",
-    "uclampge",
-    "uclampgt",
-    "uclample",
-    "uclamplt",
     "with",
     "label",
     "goto",
@@ -281,6 +306,12 @@ VALID_IR_MACROS = {
 }
 
 
+EIP_170_LIMIT = 0x6000  # 24kb
+
+SHA3_BASE = 30
+SHA3_PER_WORD = 6
+
+
 def indent(text: str, indent_chars: Union[str, List[str]] = " ", level: int = 1) -> str:
     """
     Indent lines of text in the string ``text`` using the indentation
@@ -309,6 +340,27 @@ def indent(text: str, indent_chars: Union[str, List[str]] = " ", level: int = 1)
     return "".join(indented_lines)
 
 
+def timeit(func):
+    @functools.wraps(func)
+    def timeit_wrapper(*args, **kwargs):
+        start_time = time.perf_counter()
+        result = func(*args, **kwargs)
+        end_time = time.perf_counter()
+        total_time = end_time - start_time
+        print(f"Function {func.__name__} Took {total_time:.4f} seconds")
+        return result
+
+    return timeit_wrapper
+
+
+@contextlib.contextmanager
+def timer(msg):
+    t0 = time.time()
+    yield
+    t1 = time.time()
+    print(f"{msg} took {t1 - t0}s")
+
+
 def annotate_source_code(
     source_code: str,
     lineno: int,
@@ -386,6 +438,4 @@ def annotate_source_code(
     return "\n".join(cleanup_lines)
 
 
-__all__ = [
-    "cached_property",
-]
+__all__ = ["cached_property"]
